/*
 *
 *  \file       test_check.c
 *  \brief      Tests integrity checks
 *
 *  \todo       Add SHA256
 *
 *  Author:     Lasse Collin
 *
 *  This file has been put into the public domain.
 *  You can do whatever you want with this file.
 */

#include "tests.h"


static const uint8_t test_string[9] = "123456789";
static const uint8_t test_unaligned[12] = "xxx123456789";


static bool test_crc32(void)
{
	uint32_t test_vector = UINT32_C(0xCBF43926);
	size_t i;
	uint32_t crc;
	
	/* Test 1 */
	crc = lzma_crc32(test_string, sizeof(test_string), 0);
	expect(crc == test_vector);

	/* Test 2 */
	crc = lzma_crc32(test_unaligned + 3, sizeof(test_string), 0);
	expect(crc == test_vector);

	/* Test 3 */
	crc = 0;
	for (i = 0; i < sizeof(test_string); ++i)
		crc = lzma_crc32(test_string + i, 1, crc);
	expect(crc == test_vector);

	return false;
}


static bool test_crc64(void)
{
	ll_uint64_t test_vector = LL_UINT64_C(0x995DC9BBUL, 0xDF1939FAUL);
	size_t i;
	ll_uint64_t crc;

	/* Test 1 */
	crc = lzma_crc64(test_string, sizeof(test_string), ll_uint64_zero);
	expect(ll_uint64_eq(crc, test_vector));

	/* Test 2 */
	crc = lzma_crc64(test_unaligned + 3, sizeof(test_string), ll_uint64_zero);
	expect(ll_uint64_eq(crc, test_vector));

	/* Test 3 */
	crc = ll_uint64_zero;
	for (i = 0; i < sizeof(test_string); ++i)
		crc = lzma_crc64(test_string + i, 1, crc);
	expect(ll_uint64_eq(crc, test_vector));

	return false;
}


int main(void)
{
	bool error = false;

#if defined(LZMA_SLB) && (defined(__atarist__) || defined(__TOS__))
	{
		long ret;
		if ((ret = slb_lzma_open("../atari")) < 0)
		{
			fprintf(stderr, "cannot load " LZMA_SHAREDLIB_NAME ": %ld\n", ret);
			return 1;
		}
	}
#endif

	error |= test_crc32();
	error |= test_crc64();

	return error ? 1 : 0;
}
